# FooBarQuery comment
query FooBarQuery ($after: String!) {
	# fizzList comment
	fizzList(first: 100, after: $after) {
		# nodes comment
		nodes {
			# id comment
			id
			# FooFragment comment
			... FooFragment
			# on Foo comment
			... on Foo {
				# on Foo id comment
				id
			}
			# anonymous Fragment comment
			... {
				# anonymous Fragment id comment
				id
			}
			# name comment
			name
		}
	}
}
# fragment FooFragment comment
fragment FooFragment on Foo {
	# id comment
	id
}
