# description comment
"""
Cat is best kawaii animal in the world.
meow!
""" # type comment
type Cat {
  # field description comment
  """
  Shiny brillian name.
  """
  # field comment
  name: String
  """
  Only "meow" is allowed.
  """
  speaks: String
}
# end of file comment
# end of file comment 2
