schema @schemaDirective(arg: ["val1","val2"]) {
	query: Dummy
}
extend schema @schemaExtensionDirective1(arg: ["val1","val2"]) {
	subscription: Dummy
}
extend schema @schemaExtensionDirective2(arg: ["val1","val2"]) @schemaExtensionDirective3(arg: ["val1","val2"])
directive @schemaDirective(arg: [String!]!) on SCHEMA
directive @schemaExtensionDirective1(arg: [String!]!) on SCHEMA
directive @schemaExtensionDirective2(arg: [String!]!) on SCHEMA
directive @schemaExtensionDirective3(arg: [String!]!) on SCHEMA
type Dummy {
	id: ID
}
